/*----------------------------------------------;
;	Date: Friday 4th July 2008					;
;	Author: Ruben Santa Anna Zamudio			;
;	Project: Ultrasonic Range Meter with SRF05	;
;	File Name: ultrasonic_range.c				;
;	Controller: PIC16F88						;
;	Compiler: CCS C								; 
;	Software Version: 1.0						;
;	Hardware Version: 1.0						;
;----------------------------------------------*/

// If you find errors or something that can be improved, feel free to contact me by
// mail. You can reach me at ruben at tech-freaks.net, I'll be glad to hear from you.

// Si encuentras algun error en el codigo o algo que pueda mejorarse, sientete libre
// de contactarme en el mail ruben en tech-freaks.net, Me daria gusto escuchar tus comentarios

#include "16f88.h"
#fuses XT,NOWDT,NOPROTECT,NOPUT,NOBROWNOUT,NOLVP,DEBUG
#use delay( clock=4000000 )
#use fast_io(A)
#use fast_io(B)

/*--------------------------*/
/*		TIMMING CONSTANTS	*/
/*--------------------------*/
#define _SRF05_TRIGGER_PULSE	10
#define _SHORT_DELAY			100
#define _MEDIUM_DELAY 			200
#define _LONG_DELAY 			500

/*--------------------------*/
/*		PIN DEFINITIONS		*/
/*--------------------------*/
#define _SRF05_TRIGGER	PIN_B1
#define _SRF05_ECHO		PIN_B0
#define _BUZZER_OUTPUT	PIN_B3
#define _STATUS_LED		PIN_B4
#define _BUTTON_ENTER	PIN_B7
#define _BUTTON_SELECT	PIN_B6
/* LCD DATA LINES */
#define LCD_DB4   PIN_A0
#define LCD_DB5   PIN_A1
#define LCD_DB6   PIN_A2
#define LCD_DB7   PIN_A3
/* LCD CONTROL LINES */
#define LCD_RS    PIN_A4
#define LCD_RW    PIN_B4
#define LCD_E     PIN_B2

#include "flex_lcd.c"

/*--------------------------*/
/*	Variable declarations	*/
/*--------------------------*/
int iCaptureFlag, iProgValue, i;
long lCaptureValue = 0;
float fDistance = 0;
char cTextbuffer[16];
int32 lCaptureAcc = 0;

/*---------------------*/
/* Function Prototypes */
/*---------------------*/
void vHwInitialize();
void vHwFlashLed( int, int, int );

void main()
{
	vHwInitialize();
	lcd_init();        //Initialize LCD display
	
	lcd_putc( "\f" );// Make sure LCD is clear	
	lcd_putc( "  Ultra-Sonic  " );		// Show startup message and beep
	lcd_gotoxy( 1, 2 );	
	lcd_putc( "  Sensor Demo  " );
	vHwFlashLed( 2, _SHORT_DELAY, TRUE );	
	delay_ms( 2000 );		// Show the message for two seconds
	enable_interrupts( GLOBAL );
	

	for(;;)
	{	
		lcd_putc( "\f" );			// Clear LCD display and write idle display
		lcd_putc( "Mode: Continuous" );		// Write text in line 1
		lcd_gotoxy( 1, 2 );	
		lcd_putc( "Dist. 000.0cm" );		// Place text on second line		
		iProgValue = 0;
		
		for(;;) // Menu Loop
		{
			if( input( _BUTTON_SELECT ) )
			{
				iProgValue = ( iProgValue < 2 ) ? iProgValue+1 : 0;								
				lcd_putc( "\f" );		// Clear LCD display and update as needed
				switch( iProgValue )
				{
					case 0:
					lcd_putc("Mode: Continuous");
					break;
					case 1:
					lcd_putc("Mode: Average");
					break;
					case 2:
					lcd_putc("Mode: Single");
					break;
				}				
				sprintf( cTextBuffer, "Dist. 000.0cm" );	//Keep the same text on the second line
				lcd_put_string_xy( cTextBuffer, 1, 2 );
				delay_ms( 300 );
				
			}		
			if( input( _BUTTON_ENTER ) )
			{
				delay_ms( 200 );		// If the "select" key is pressed, we execute the selected program		
				break;
			}
		}	// End of menu loop
					
		switch( iProgValue )
		{
			case 0:		// Continuous sensing
				while( !input( _BUTTON_SELECT ) )
				{
					output_high( _SRF05_TRIGGER );		
					delay_us( _SRF05_TRIGGER_PULSE );	// Send SRF05 trigger Pulse
					output_low( _SRF05_TRIGGER );
					while( !iCaptureFlag )				// Wait until CCP interrupt ocurs
					iCaptureFlag = FALSE;
					fDistance = (float)lCaptureValue / 58.0;
					sprintf( cTextBuffer, "Dist. %05.1f cm", fDistance );
					lcd_put_string_xy( cTextBuffer, 1, 2 );
					delay_ms( 50 );
				}
			break;
			case 1:		// Continuous sensing with average display
				while( !input( _BUTTON_SELECT ) )
				{
					lCaptureAcc = 0;
					for( i = 0; i<10; i++ )					// Repeat ten times then output average to LCD
					{
						output_high( _SRF05_TRIGGER );		
						delay_us( _SRF05_TRIGGER_PULSE );	// Send SRF05 Trigger Pulse
						output_low( _SRF05_TRIGGER );
						while( !iCaptureFlag )				// Wait until CCP interrupt ocurs
						iCaptureFlag = FALSE;
						lCaptureAcc += lCaptureValue;						
					}
					fDistance = (float)( lCaptureAcc / 10 ) / 58.0;
					sprintf( cTextBuffer, "Dist. %05.1f cm", fDistance );
					lcd_put_string_xy( cTextBuffer, 1, 2 );
					delay_ms( 50 );
				}
			break;
			case 2:		// Single reading
				while( !input( _BUTTON_SELECT ) )
				{
					if( input( _BUTTON_ENTER ) )
					{
						output_high( _SRF05_TRIGGER );		
						delay_us( _SRF05_TRIGGER_PULSE );	// Send SRF05 Trigger Pulse
						output_low( _SRF05_TRIGGER );
						while( !iCaptureFlag )				// Wait until CCP interrupt ocurs
						iCaptureFlag = FALSE;
						fDistance = (float)lCaptureValue / 58.0;
						sprintf( cTextBuffer, "Dist. %05.1f cm", fDistance );
						lcd_put_string_xy( cTextBuffer, 1, 2 );
						delay_ms( 200 );	// Pause between readings
						while ( input ( _BUTTON_ENTER ) );		// Only one reading per keypress
					}
				}
			break;
		}
	}
}

void vHwInitialize()
{
	/* Hardware Setup Procedure */
	setup_adc_ports( NO_ANALOGS|VSS_VDD );
	setup_adc( ADC_OFF );
	setup_timer_0( RTCC_INTERNAL | RTCC_DIV_64 );
	setup_timer_1( T1_INTERNAL|T1_DIV_BY_1 );
	setup_timer_2( T2_DISABLED,0,1 );
	setup_comparator( NC_NC_NC_NC );
	setup_vref( FALSE );
	
	/*Configure Input Capture */	
	setup_ccp1(CCP_CAPTURE_RE);
		
	/* Configure I/O port direction */
	SET_TRIS_A( 0b00000000 );
	SET_TRIS_B( 0b11000001 );
		
	/* Enable interrupts */
	enable_interrupts( INT_CCP1 );

	/* Set outputs to a known state */
	output_a( 0 );
	output_b( 0 );	
	return;
}

void vHwFlashLed( int iLedFlashes, int iDelayMs, int iBeep )
{
	int i;
	for(i=0; i< iLedFlashes; i++)
	{
		if( iBeep )
		output_high( _BUZZER_OUTPUT );
		output_high( _STATUS_LED );
		delay_ms( iDelayMs );
		output_low( _STATUS_LED );
		if( iBeep )
		output_low( _BUZZER_OUTPUT );
		delay_ms( iDelayMs );
	}  
}

#int_ccp1
void vCaptureInterrupt()
{
	if( input( _SRF05_ECHO ))		// If echo pin is high, we prepare the timer and CCP module
	{				
		setup_ccp1( CCP_CAPTURE_FE );	// To capture the falling edge of the echo pulse
		SET_TIMER1( 0 );
	} 
	else
	{
		setup_ccp1( CCP_CAPTURE_RE );	// Capture the rising edge next time
		lCaptureValue = CCP_1;			// Capture the timer value
		iCaptureFlag = TRUE;
	}
}	                 
